(*
Test program for Example algorithms in Pascal by Tom Swan from
Algorithm Alley in DDJ. This program does not draw a ruler--but it can
do so by adding appopriate graphics commands. For test purposes, the
program simply writes the ruler marking coordinates to the display.
See the C++ RULER example, written in Borland C++ 4.0 for Windows,
for a complete implementation.
*)

program test;

const
  smallMarkerIncr = 2;
  stackMax = 100;

var
  Top: Integer;
  Stack: array[0 .. stackMax] of Integer;
  StackIndex: Integer;

procedure Line(X1, Y1, X2, Y2: Integer);
begin
  Writeln('(', X1, ',', Y1, ')-(', X2, ',', Y2, ')');
end;

procedure Throw(S: String);
begin
  Writeln(S);
  Halt;
end;

function StackEmpty: Boolean;
begin
  StackEmpty := StackIndex <= 0;
end;

procedure PushInt(N: Integer);
begin
  StackIndex := StackIndex + 1;
  if StackIndex > stackMax then
    Throw('Stack overflow');
  Stack[StackIndex] := N;
end;

function PopInt: Integer;
begin
  if StackEmpty then
    Throw('Stack underflow');
  PopInt := Stack[StackIndex];
  StackIndex := StackIndex - 1;
end;

procedure Push(L, R, M, H, Level: Integer);
begin
  PushInt(L);
  PushInt(R);
  PushInt(M);
  PushInt(H);
  PushInt(Level);
end;

procedure Pop(var L, R, M, H, Level: Integer);
begin
  Level := PopInt;
  H := PopInt;
  M := PopInt;
  R := PopInt;
  L := PopInt;
end;

(*
{ Ruler implementation #1 (recursive version) }
procedure Ruler(L, R, H, Level: Integer);
var
  M: Integer;
begin
  if H < 0 then
    Throw('Levels incomplete');
  if Level > 0 then
  begin
    M := (L + R) DIV 2;
    Line(M, Top, M, Top + H);
    Ruler(L, M, H - smallMarkerIncr, Level - 1);
    Ruler(M, R, H - smallMarkerIncr, Level - 1);
  end;
end;
*)

(*
{ Ruler implementation #2 (end-recursion removed using goto) }
procedure Ruler(L, R, H, Level: Integer);
label
  RulerStart, RulerEnd;
var
  M: Integer;
begin
RulerStart:
  if H <= 0 then
    Throw('Levels incomplete');
  if Level <= 0 then
    goto RulerEnd;
  M := (L + R) DIV 2;
  Line(M, Top, M, Top + H);
  Ruler(L, M, H - smallMarkerIncr, Level - 1);
  L := M;
  Level := Level - 1;
  H := H - smallMarkerIncr;
  goto RulerStart;
RulerEnd:
end;
*)

(*
{ Ruler implementation #3 (end-recursion removed using while) }
procedure Ruler(L, R, H, Level: Integer);
var
  M: Integer;
begin
  while level > 0 do
  begin
    if H <= 0 then
      Throw('Levels incomplete');
    M := (L + R) DIV 2;
    Line(M, Top, M, Top + H);
    Ruler(L, M, H - smallMarkerIncr, Level - 1);
    L := M;
    Level := Level - 1;
    H := H - smallMarkerIncr;
  end;
end;
*)

(*
{ Ruler implementation #4 (all recursion removed using goto) }
procedure Ruler(L, R, H, Level: Integer);
label
  RulerStart, RulerRestart, RulerEnd;
var
  M: Integer;
begin
RulerStart:
  if Level = 0 then
    goto RulerEnd;
  if H <= 0 then
    Throw('Levels incomplete');
  M := (L + R) DIV 2;
  Line(M, Top, M, Top + H);
  Push(L, R, M, H, Level);
  L := L;
  R := M;
  H := H - smallMarkerIncr;
  Level := Level - 1;
  goto RulerStart;
RulerRestart:
  Pop(L, R, M, H, Level);
  L := M;
  Level := Level - 1;
  H := H - smallMarkerIncr;
  goto RulerStart;
RulerEnd:
  if not StackEmpty then
    goto RulerRestart;
end;
*)

(*
{ Ruler implementation #5 (all recursion removed using while and if) }
procedure Ruler(L, R, H, Level: Integer);
var
  M: Integer;
begin
  Push(L, R, 0, H, Level);
  while not StackEmpty do
  begin
    Pop(L, R, M, H, Level);
    while Level > 0 do
    begin
      if H <= 0 then
        Throw('Levels incomplete');
      M := (L + R) DIV 2;
      Line(M, Top, M, Top + H);
      Push(L, R, M, H, Level);
      L := L;
      R := M;
      H := H - smallMarkerIncr;
      Level := Level - 1;
    end;
    if not StackEmpty then
    begin
      Pop(L, R, M, H, Level);
      L := M;
      Level := Level - 1;
      H := H - smallMarkerIncr;
      Push(L, R, M, H, Level);
    end;
  end;
end;
*)

{ Ruler implementation #6 (final optimized version) }
procedure Ruler(L, R, H, Level: Integer);
var
  M: Integer;
begin
  Push(L, R, 0, H, Level);
  while not StackEmpty do
  begin
    Pop(L, R, M, H, Level);
    while Level > 0 do
    begin
      if H <= 0 then
        Throw('Levels incomplete');
      M := (L + R) DIV 2;
      Line(M, Top, M, Top + H);
      H := H - smallMarkerIncr;
      Level := Level - 1;
      Push(M, R, M, H, Level);
      R := M;
    end;
  end;
end;

begin
  Top := 3;
  StackIndex := 0;
  Ruler(3, 103, 10, 4);
end.
